package io.tswf.minecraft.blockventure.core.transmutation

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.AtExit
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.AtFieldCall
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.ClassReference
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.FieldValue
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.MethodDescription
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Offset
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Transmute
import io.tswf.groovy.bettergroovy.asm.transmutation.injection.points.FieldCallType
import io.tswf.groovy.bettergroovy.asm.transmutation.meta.MissingTargetStrategy
import io.tswf.minecraft.blockventure.core.network.packet.ClientNetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.network.packet.NetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.network.packet.NetworkPlayPacketWrapperCreator
import io.tswf.minecraft.blockventure.core.network.packet.ServerNetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.view.ViewNetworkState
import io.tswf.minecraft.blockventure.core.view.ViewNetworkStatePacketHandler
import net.minecraft.network.NetworkSide
import net.minecraft.network.NetworkState
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.packet.Packet

import java.util.function.Function

@Slf4j
@CompileStatic
class TransmutationNetworkState {

    @Transmute(
            targetClass = @ClassReference(NetworkState),
            targetMethod = @MethodDescription(
                    name = "<clinit>",
                    descriptor = "()V"
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtFieldCall(
            callType = FieldCallType.STATIC_SET,
            type = @ClassReference(NetworkState),
            name = "PLAY",
            owner = @ClassReference(NetworkState)
    )
    static void registerBlockventurePlayNetworkPackets() {
        def playNetworkState = NetworkState.PLAY
        def viewPlayNetworkState = (ViewNetworkState) (Object) playNetworkState
        def packetHandlers = (Map<NetworkSide, ViewNetworkStatePacketHandler>) (Map) viewPlayNetworkState.packetHandlers

        for (def networkSide : NetworkSide.values()) {
            packetHandlers[networkSide]?.let { viewPacketHandler ->
                log.info("Registering ${ClientNetworkPlayPacketWrapper.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        ClientNetworkPlayPacketWrapper as Class,
                        { PacketByteBuf buf -> NetworkPlayPacketWrapperCreator.createWrapperForPlayPacket(buf, false) } as Function<PacketByteBuf, Packet>
                )

//                packetTypeToStateMap[ClientNetworkPlayPacketWrapper] = NetworkState.PLAY

                log.info("Registering ${ServerNetworkPlayPacketWrapper.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        ServerNetworkPlayPacketWrapper as Class,
                        { PacketByteBuf buf -> NetworkPlayPacketWrapperCreator.createWrapperForPlayPacket(buf, true) } as Function<PacketByteBuf, Packet>
                )

//                packetTypeToStateMap[ServerNetworkPlayPacketWrapper] = NetworkState.PLAY
            }
        }
    }
}
